#include <stdio.h>

#include "../include/uCTest/uCTest.h"

#define XSTR( X ) UCTEST_XSTR( X )

#if defined( CONFIG_OUTPUT_NONE )
const __flash char uctest_eqFailMsg[] = "";

const __flash char uctest_nearFailMsg[] = "";

const __flash char uctest_simpleFailMsg[] = "";

const __flash char uctest_deathFailMsg[] = "";

void uctest_printRuntimeStats( int suites, int tests ) {
    ( void ) suites;
    ( void ) tests;
}

void uctest_printRuntimeStatsEnd( int suites, int tests, unsigned long cycles ) {
    ( void ) suites;
    ( void ) tests;
    ( void ) cycles;
}

void uctest_printStartup( int suites, int tests ) {
    ( void ) suites;
    ( void ) tests;
}

void uctest_printGlobalSetup( void ) {
}

void uctest_printGlobalTeardown( void ) {
}

void uctest_printSuiteStats( const __flash char *suite, int tests ) {
    ( void ) suite;
    ( void ) tests;
}

void uctest_printSuiteStatsBegin( const __flash char *suite, int tests ) {
    ( void ) suite;
    ( void ) tests;
}

void uctest_printSuiteStatsEnd( const __flash char *suite, int tests, int failed,
                                unsigned long cycles ) {
    ( void ) suite;
    ( void ) tests;
    ( void ) failed;
    ( void ) cycles;
}

void uctest_printTest( const __flash char *mode, const __flash struct uctest_test *test ) {
    ( void ) mode;
    ( void ) test;
}

void uctest_printTestBegin( const __flash struct uctest_test *test ) {
    ( void ) test;
}

void uctest_printTestSuccess( const __flash struct uctest_test *test, unsigned long cycles ) {
    ( void ) test;
    ( void ) cycles;
}

void uctest_printAllTestsEnd( int testsSuccess, int testsFailure ) {
    ( void ) testsSuccess;
    ( void ) testsFailure;
}

void uctest_printFailedTestsAtEnd( int tests ) {
    ( void ) tests;
}

void uctest_printTestFailure( const __flash struct uctest_test *test, unsigned long cycles ) {
    ( void ) test;
    ( void ) cycles;
}

void uctest_printFailureLocation( const char *file, int line ) {
    ( void ) file;
    ( void ) line;
}

void uctest_printUserMessage( const __flash char *fmt, va_list args ) {
    ( void ) fmt;
    ( void ) args;
}
#elif defined( CONFIG_OUTPUT_JSON )
void uctest_printRuntimeStats( int suites, int tests ) {
    ( void ) suites;
    ( void ) tests;
}

void uctest_printRuntimeStatsEnd( int suites, int tests, unsigned long cycles ) {
    printf_P( XSTR( "    {}\n  ],\n  \"time\": %lu,\n" ), cycles );
    ( void ) suites;
    ( void ) tests;
}

void uctest_printStartup( int suites, int tests ) {
    ( void ) suites;
    ( void ) tests;
    printf_P( XSTR( "{\n  \"name\": \"AllTests\",\n  \"testsuites\": [\n" ), tests );
}

void uctest_printGlobalSetup( void ) {
}

void uctest_printGlobalTeardown( void ) {
}

void uctest_printSuiteStats( const __flash char *suite, int tests ) {
    ( void ) suite;
    ( void ) tests;
}

void uctest_printSuiteStatsBegin( const __flash char *suite, int tests ) {
    printf_P( XSTR( "    {\n      \"name\": \"%S\",\n      \"testsuite\": [\n" ), suite, tests );
}

void uctest_printSuiteStatsEnd( const __flash char *suite, int tests, int failed,
                                unsigned long cycles ) {
    printf_P(
        XSTR(
            "        {}\n      ],\n      \"time\": %lu,\n      \"tests\": %d,\n      \"failures\": "
            "%d\n    },\n" ),
        cycles, tests, failed );
    ( void ) suite;
}

void uctest_printTest( const __flash char *mode, const __flash struct uctest_test *test ) {
    ( void ) mode;
    ( void ) test;
}

static int test_failed;
void uctest_printTestBegin( const __flash struct uctest_test *test ) {
    test_failed = 0;
    printf_P( XSTR( "        {\n          \"classname\": \"%S\",\n          \"name\": \"%S\",\n    "
                    "      \"status\": \"RUN\",\n" ),
              test->suite_name, test->test_name );
}

void uctest_printTestSuccess( const __flash struct uctest_test *test, unsigned long cycles ) {
    printf_P( XSTR( "          \"result\": \"COMPLETED\",\n          \"time\": %lu\n        },\n" ),
              cycles );
    ( void ) test;
}

void uctest_printTestFailure( const __flash struct uctest_test *test, unsigned long cycles ) {
    if( cycles ) {
        printf_P( XSTR( "            {}\n          ],\n" ) );
        uctest_printTestSuccess( test, cycles );
    }
}

void uctest_printAllTestsEnd( int testsSuccess, int testsFailure ) {
    printf_P( XSTR( "  \"tests\": %d,\n  \"failures\": %d\n}\n" ), ( testsSuccess + testsFailure ),
              testsFailure );
}

void uctest_printFailedTestsAtEnd( int tests ) {
    ( void ) tests;
}

static int inString = 0;
void uctest_printFailureLocation( const char *file, int line ) {
    if( !test_failed ) {
        test_failed = !test_failed;
        printf_P( XSTR( "          \"failures\": [\n" ) );
    }

    printf_P( XSTR( "            {\n              \"failure\": \"" ), file, line );
    inString = 1;
    printf_P( XSTR( "%s:%d\n" ), file, line );
}

void uctest_printUserMessage( const __flash char *fmt, va_list args ) {
    vfprintf_P( stdout, fmt, args );
    inString = 0;

    printf_P( XSTR( "\",\n              \"type\": \"\"\n            },\n" ) );
}

int uctest_shouldEscape( char c ) {
    switch( c ) {
        case '\"':
        case '\'':
        case '\\':
        case '\n':
        case '/':
        case '\t': return inString;
        default: return 0;
    }
}
#else
void uctest_printRuntimeStats( int suites, int tests ) {
    printf_P( XSTR( "\e[32m[==========]\e[0m Running %d tests from %d test suites.\n" ), tests,
              suites );
}

void uctest_printRuntimeStatsEnd( int suites, int tests, unsigned long cycles ) {
    printf_P(
        XSTR( "\e[32m[==========]\e[0m %d tests from %d test suites ran. (%lu cycles total)\n" ),
        suites, tests, cycles );
}

void uctest_printStartup( int suites, int tests ) {
    printf_P( XSTR( "Running main\n" ) );
    uctest_printRuntimeStats( suites, tests );
}

void uctest_printGlobalSetup( void ) {
    printf_P( XSTR( "\e[32m[----------]\e[0m Global test environment set-up.\n" ) );
}

void uctest_printGlobalTeardown( void ) {
    printf_P( XSTR( "\e[32m[----------]\e[0m Global test environment tear-down.\n" ) );
}

void uctest_printSuiteStats( const __flash char *suite, int tests ) {
    printf_P( XSTR( "\e[32m[----------]\e[0m %d tests from %S" ), tests, suite );
}

void uctest_printSuiteStatsBegin( const __flash char *suite, int tests ) {
    uctest_printSuiteStats( suite, tests );
    printf( "\n" );
}

void uctest_printSuiteStatsEnd( const __flash char *suite, int tests, int failed,
                                unsigned long cycles ) {
    ( void ) failed;
    uctest_printSuiteStats( suite, tests );
    printf_P( XSTR( " (%lu cycles)\n" ), cycles );
    printf( "\n" );
}

void uctest_printTest( const __flash char *mode, const __flash struct uctest_test *test ) {
    printf_P( XSTR( "\e[32m[ %S ]\e[0m %S.%S" ), mode, test->suite_name, test->test_name );
}

void uctest_printTestBegin( const __flash struct uctest_test *test ) {
    uctest_printTest( XSTR( "RUN     " ), test );
    printf( "\n" );
}

void uctest_printTestSuccess( const __flash struct uctest_test *test, unsigned long cycles ) {
    uctest_printTest( XSTR( "      OK" ), test );
    printf_P( XSTR( " (%lu cycles)\n" ), cycles );
}

void uctest_printFailedTestsAtEnd( int tests ) {
    printf_P( XSTR( "\n%d FAILED TESTS\n" ), tests );
}

void uctest_printAllTestsEnd( int testsSuccess, int testsFailure ) {
    printf_P( XSTR( "\e[32m[  PASSED  ]\e[0m %d tests.\n" ), testsSuccess );
    if( testsFailure ) {
        printf_P( XSTR( "\e[31m[  FAILED  ]\e[0m %d tests, listed below:\n" ), testsFailure );
    }
}

void uctest_printTestFailure( const __flash struct uctest_test *test, unsigned long cycles ) {
    printf_P( XSTR( "\e[31m[  FAILED  ]\e[0m %S.%S" ), test->suite_name, test->test_name );
    printf_P( XSTR( " (%lu cycles)\n" ), cycles );
}

void uctest_printFailureLocation( const char *file, int line ) {
    printf_P( XSTR( "%s:%d: Failure\n" ), file, line );
}

void uctest_printUserMessage( const __flash char *fmt, va_list args ) {
    vfprintf_P( stdout, fmt, args );
}
#endif

#if !defined( CONFIG_OUTPUT_NONE )
const __flash char uctest_eqFailMsg[] =
    "Expected equality of these values :\n %s\n   Which is : %d\n %s\n   Which is : %d\n";

const __flash char uctest_nearFailMsg[] =
    "The difference between %s and %s is %d, which exceeds %s, where\n  %s "
    "evaluates to %d\n  %s evaluates to %d, and\n  %s evaluates to %d.\n";

const __flash char uctest_simpleFailMsg[] = "Expected: %s, actual: %d vs %d\n";

const __flash char uctest_deathFailMsg[] =
    "Death test: %s\n    Result: failed to die.\n[  DEATH   ]\n";
#endif
