#!/usr/bin/env python
# Python program to demonstrate
# Conversion of JSON data to
# dictionary

# importing the module
import json
import sys
from xml.dom import getDOMImplementation
from xml.sax.saxutils import escape

def main():
    if len(sys.argv) < 3:
        file = sys.stdin
    else:
        file = open(sys.argv[2], 'r')

    with file as json_file:
        data = json.load(json_file)
        dom = getDOMImplementation()

        document = dom.createDocument(None, "testsuites", None)
        root = document.documentElement

        root.setAttribute("name", data['name'])
        root.setAttribute("tests", str(data['tests']))
        root.setAttribute("errors", str(0))
        root.setAttribute("failures", str(data['failures']))
        root.setAttribute("time", str(data['time']))

        for suite in data['testsuites']:
            if 'name' in suite:
                suite_element = document.createElement("testsuite")

                suite_element.setAttribute("name", suite['name'])
                suite_element.setAttribute("tests", str(suite['tests']))
                suite_element.setAttribute("errors", str(0))
                suite_element.setAttribute("failures", str(suite['failures']))
                suite_element.setAttribute("time", str(suite['time']))

                for testcase in suite['testsuite']:
                    if 'name' in testcase:
                        tc_element = document.createElement("testcase")

                        tc_element.setAttribute("name", testcase['name'])
                        tc_element.setAttribute("status", testcase['status'])
                        tc_element.setAttribute("result", testcase['result'])
                        tc_element.setAttribute("time", str(testcase['time']))
                        tc_element.setAttribute("classname", testcase['classname'])

                        if 'failures' in testcase:
                            for failure in testcase['failures']:
                                if 'failure' in failure:
                                    failure_element = document.createElement("failure")

                                    fail_str = document.createCDATASection(failure['failure'])
                                    failure_element.setAttribute( "message", escape(failure['failure'], entities={"\n": "&#x0A;", }))

                                    failure_element.appendChild( fail_str )
                                    tc_element.appendChild(failure_element)

                        suite_element.appendChild(tc_element)

                root.appendChild( suite_element )


        with open(sys.argv[1], mode='w') as fh:
            fh.write(document.toxml('utf-8').decode('utf-8'))

    sys.exit( 0 )

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print("Usage: " + sys.argv[0] + " <output> [<input-files> or stdin]")
        sys.exit(1)

    main()
